@@ -52,7 +52,7 @@ struct SecureDhtConfig;
 class OPENDHT_PUBLIC DhtRunner {
 
 public:
-    typedef std::function<void(NodeStatus, NodeStatus)> StatusCallback;
+    using StatusCallback = std::function<void(NodeStatus, NodeStatus)>;
 
     struct Config {
         SecureDhtConfig dht_config;
@@ -63,6 +63,14 @@ public:
         bool peer_publish;
     };
 
+    struct Context {
+        std::unique_ptr<Logger> logger {};
+        std::shared_ptr<PeerDiscovery> peerDiscovery {};
+        StatusCallback statusChangedCallback {};
+        CertificateStoreQuery certificateStore {};
+        Context() {}
+    };
+
     DhtRunner();
     virtual ~DhtRunner();
 
@@ -316,7 +324,8 @@ public:
 
     std::vector<ValuesExport> exportValues() const;
 
-    void setLoggers(LogMethod err = NOLOG, LogMethod warn = NOLOG, LogMethod debug = NOLOG);
+    void setLogger(const Logger& logger = {});
+    void setLoggers(LogMethod err = {}, LogMethod warn = {}, LogMethod debug = {});
 
     /**
      * Only print logs related to the given InfoHash (if given), or disable filter (if zeroes).
@@ -375,7 +384,7 @@ public:
             /*.peer_publish = */true,
         });
     }
-    void run(in_port_t port, const Config& config);
+    void run(in_port_t port, const Config& config, Context&& context = {});
 
     /**
      * @param local4: Local IPv4 address and port to bind. Can be null.
@@ -385,12 +394,12 @@ public:
      * @param threaded: If false, loop() must be called periodically. Otherwise a thread is launched.
      * @param cb: Optional callback to receive general state information.
      */
-    void run(const SockAddr& local4, const SockAddr& local6, const Config& config);
+    void run(const SockAddr& local4, const SockAddr& local6, const Config& config, Context&& context = {});
 
     /**
      * Same as @run(sockaddr_in, sockaddr_in6, Identity, bool, StatusCallback), but with string IP addresses and service (port).
      */
-    void run(const char* ip4, const char* ip6, const char* service, const Config& config);
+    void run(const char* ip4, const char* ip6, const char* service, const Config& config, Context&& context = {});
 
     void setOnStatusChanged(StatusCallback&& cb) {
         statusCb = std::move(cb);
@@ -424,6 +433,8 @@ public:
      */
     void join();
 
+    std::shared_ptr<PeerDiscovery> getPeerDiscovery() const { return peerDiscovery_; };
+
     void setProxyServer(const std::string& proxy, const std::string& pushNodeId = "");
 
     /**
@@ -539,8 +550,7 @@ private:
     std::string pushToken_;
 
     /** PeerDiscovery Parameters */
-    std::unique_ptr<PeerDiscovery> peerDiscovery4_;
-    std::unique_ptr<PeerDiscovery> peerDiscovery6_;
+    std::shared_ptr<PeerDiscovery> peerDiscovery_;
 };
 
 }
